import SimpleOpenNI.*;     // simple-openni
import saito.objloader.*;  // OBJローダ

SimpleOpenNI kinect;        // Kinect
OBJModel     model;         // 3Dモデル
int          cntlUser = 0;  // 操作を行うユーザのID
float        rotX;          // 3Dモデルの回転角度(X軸)
float        rotY;          // 3Dモデルの回転角度(Y軸)

void setup() {
  size(640, 480, P3D);  // 画面のサイズ設定（3D空間を扱うのでP3Dを付けます）
  
  // 3Dモデルのロード
  model = new OBJModel(this, "cassini.obj", "relative", TRIANGLES);
  model.scale(8);
  noStroke();

  // Kinectに関する設定
  kinect = new SimpleOpenNI(this);
  kinect.setMirror(true);
  kinect.enableDepth();
  kinect.enableRGB();
  kinect.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);
  kinect.alternativeViewPointDepthToImage();
}

void draw() {  
  background(32);                            // 背景のリセット
  kinect.update();                           // Kinectのデータ更新
  image(kinect.rgbImage(), 0, 0, 160, 120);  // 画面左上に小さくカラー画像を表示

  // 骨格がトラッキングできていたら3Dモデルの操作を行う
  if ( kinect.isTrackingSkeleton(cntlUser) ) {
    // 右手と右肩の3次元位置を取得
    PVector hand = new PVector();
    PVector shoulder = new PVector();
    kinect.getJointPositionSkeleton(cntlUser, SimpleOpenNI.SKEL_RIGHT_HAND, hand);
    kinect.getJointPositionSkeleton(cntlUser, SimpleOpenNI.SKEL_RIGHT_SHOULDER, shoulder);

    // 手と肩の距離から3Dモデルの回転量を決定
    rotY += (hand.x - shoulder.x) * 0.0006;  // 左右方向
    rotX += (hand.y - shoulder.y) * 0.0006;  // 上下方向
  }

  lights();                         // 照明の設定
  translate(width/2, height/2, 0);  // 画面中央に配置
  rotateX(rotX);                    // X軸まわりの回転
  rotateY(rotY);                    // Y軸まわりの回転
  model.draw();                     // 3Dモデルの描画
}

// 新しいユーザを見つけた場合の処理
void onNewUser(int userId) {
  kinect.requestCalibrationSkeleton(userId, true);  // 骨格キャリブレーションを行う
}

// キャリブレーション終了時の処理
void onEndCalibration(int userId, boolean successfull) {
  if (successfull) { 
    kinect.startTrackingSkeleton(userId);  // 骨格トラッキングを開始
    cntlUser = userId;                     // 操作を行うユーザを決定
  }
}

